package com.ustadmobile.core.db.dao

import com.ustadmobile.door.DoorDatabase
import com.ustadmobile.door.DoorDbType
import com.ustadmobile.door.EntityInsertionAdapter
import com.ustadmobile.door.PreparedStatementConfig
import com.ustadmobile.door.ext.prepareAndUseStatement
import com.ustadmobile.door.ext.prepareAndUseStatementAsync
import com.ustadmobile.door.ext.useResults
import com.ustadmobile.door.jdbc.PreparedStatement
import com.ustadmobile.door.jdbc.ext.executeUpdateAsyncKmp
import com.ustadmobile.lib.db.entities.StateContentEntity
import kotlin.Boolean
import kotlin.IllegalArgumentException
import kotlin.Long
import kotlin.String
import kotlin.Unit
import kotlin.collections.List

public class StateContentDao_JdbcKt(
  public val _db: DoorDatabase
) : StateContentDao() {
  public val _insertAdapterStateContentEntity_: EntityInsertionAdapter<StateContentEntity> = object
      : EntityInsertionAdapter<StateContentEntity>(_db) {
    public override fun makeSql(returnsId: Boolean) = when(dbType) {
      DoorDbType.SQLITE -> {
        "INSERT INTO StateContentEntity (stateContentUid, stateContentStateUid, stateContentKey, stateContentValue, isIsactive, stateContentMasterChangeSeqNum, stateContentLocalChangeSeqNum, stateContentLastChangedBy, stateContentLct) VALUES(?, ?, ?, ?, ?, ?, ?, ?, ?)"
      }
      DoorDbType.POSTGRES ->  {
        "INSERT INTO StateContentEntity (stateContentUid, stateContentStateUid, stateContentKey, stateContentValue, isIsactive, stateContentMasterChangeSeqNum, stateContentLocalChangeSeqNum, stateContentLastChangedBy, stateContentLct) VALUES(COALESCE(?,nextval('StateContentEntity_stateContentUid_seq')), ?, ?, ?, ?, ?, ?, ?, ?)" + if(returnsId) { " RETURNING stateContentUid" } else "" 
      }
      else -> {
        throw IllegalArgumentException("Unsupported db type")
      }
    }

    public override fun bindPreparedStmtToEntity(stmt: PreparedStatement,
        entity: StateContentEntity): Unit {
      if(entity.stateContentUid == 0L) {
        stmt.setObject(1, null)
      } else {
        stmt.setLong(1, entity.stateContentUid)
      }
      stmt.setLong(2, entity.stateContentStateUid)
      stmt.setString(3, entity.stateContentKey)
      stmt.setString(4, entity.stateContentValue)
      stmt.setBoolean(5, entity.isIsactive)
      stmt.setLong(6, entity.stateContentMasterChangeSeqNum)
      stmt.setLong(7, entity.stateContentLocalChangeSeqNum)
      stmt.setInt(8, entity.stateContentLastChangedBy)
      stmt.setLong(9, entity.stateContentLct)
    }
  }

  public override suspend fun replicateOnNewNode(newNodeId: Long): Unit {
    val _stmtConfig = PreparedStatementConfig("""
    |
    |     REPLACE INTO StateContentEntityReplicate(scePk, sceDestination)
    |      SELECT DISTINCT StateContentEntity.stateContentUid AS scePk,
    |             ? AS sceDestination
    |        FROM StateContentEntity
    |       WHERE StateContentEntity.stateContentLct != COALESCE(
    |             (SELECT sceVersionId
    |                FROM StateContentEntityReplicate
    |               WHERE scePk = StateContentEntity.stateContentUid
    |                 AND sceDestination = ?), 0) 
    |      /*psql ON CONFLICT(scePk, sceDestination) DO UPDATE
    |             SET scePending = true
    |      */       
    |    
    """.trimMargin() , postgreSql = """
    |INSERT INTO StateContentEntityReplicate(scePk, sceDestination)
    |      SELECT DISTINCT StateContentEntity.stateContentUid AS scePk,
    |             ? AS sceDestination
    |        FROM StateContentEntity
    |       WHERE StateContentEntity.stateContentLct != COALESCE(
    |             (SELECT sceVersionId
    |                FROM StateContentEntityReplicate
    |               WHERE scePk = StateContentEntity.stateContentUid
    |                 AND sceDestination = ?), 0) 
    |       ON CONFLICT(scePk, sceDestination) DO UPDATE
    |             SET scePending = true
    |             
    |    
    |""".trimMargin())
    _db.prepareAndUseStatementAsync(_stmtConfig) {
      _stmt ->
      _stmt.setLong(1, newNodeId)
      _stmt.setLong(2, newNodeId)
      val _numUpdates = _stmt.executeUpdateAsyncKmp()
    }
  }

  public override suspend fun replicateOnChange(): Unit {
    val _stmtConfig = PreparedStatementConfig("""
    |
    | REPLACE INTO StateContentEntityReplicate(scePk, sceDestination)
    |  SELECT DISTINCT StateContentEntity.stateContentUid AS sceUid,
    |         UserSession.usClientNodeId AS sceDestination
    |    FROM ChangeLog
    |         JOIN StateContentEntity
    |             ON ChangeLog.chTableId = 72
    |                AND ChangeLog.chEntityPk = StateContentEntity.stateContentUid
    |         JOIN UserSession ON UserSession.usStatus = 1
    |   WHERE UserSession.usClientNodeId != (
    |         SELECT nodeClientId 
    |           FROM SyncNode
    |          LIMIT 1)
    |     AND StateContentEntity.stateContentLct != COALESCE(
    |         (SELECT sceVersionId
    |            FROM StateContentEntityReplicate
    |           WHERE scePk = StateContentEntity.stateContentUid
    |             AND sceDestination = UserSession.usClientNodeId), 0)
    | /*psql ON CONFLICT(scePk, sceDestination) DO UPDATE
    |     SET scePending = true
    |  */               
    | 
    """.trimMargin() , postgreSql = """
    |INSERT INTO StateContentEntityReplicate(scePk, sceDestination)
    |  SELECT DISTINCT StateContentEntity.stateContentUid AS sceUid,
    |         UserSession.usClientNodeId AS sceDestination
    |    FROM ChangeLog
    |         JOIN StateContentEntity
    |             ON ChangeLog.chTableId = 72
    |                AND ChangeLog.chEntityPk = StateContentEntity.stateContentUid
    |         JOIN UserSession ON UserSession.usStatus = 1
    |   WHERE UserSession.usClientNodeId != (
    |         SELECT nodeClientId 
    |           FROM SyncNode
    |          LIMIT 1)
    |     AND StateContentEntity.stateContentLct != COALESCE(
    |         (SELECT sceVersionId
    |            FROM StateContentEntityReplicate
    |           WHERE scePk = StateContentEntity.stateContentUid
    |             AND sceDestination = UserSession.usClientNodeId), 0)
    |  ON CONFLICT(scePk, sceDestination) DO UPDATE
    |     SET scePending = true
    |                 
    | 
    |""".trimMargin())
    _db.prepareAndUseStatementAsync(_stmtConfig) {
      _stmt ->
      val _numUpdates = _stmt.executeUpdateAsyncKmp()
    }
  }

  public override fun findAllStateContentWithStateUid(id: Long): List<StateContentEntity> {
    var _result = mutableListOf<com.ustadmobile.lib.db.entities.StateContentEntity>()
    val _stmtConfig =
        PreparedStatementConfig("SELECT * FROM StateContentEntity WHERE stateContentStateUid = ? AND isIsactive"
        , postgreSql = """
    |SELECT * FROM StateContentEntity WHERE stateContentStateUid = ? AND isIsactive
    |""".trimMargin())
    _db.prepareAndUseStatement(_stmtConfig) {
      _stmt ->
      _stmt.setLong(1, id)
      _stmt.executeQuery().useResults {
         _resultSet ->
        while(_resultSet.next()) {
          val tmp_stateContentUid = _resultSet.getLong("stateContentUid")
          val tmp_stateContentStateUid = _resultSet.getLong("stateContentStateUid")
          val tmp_stateContentKey = _resultSet.getString("stateContentKey")
          val tmp_stateContentValue = _resultSet.getString("stateContentValue")
          val tmp_isIsactive = _resultSet.getBoolean("isIsactive")
          val tmp_stateContentMasterChangeSeqNum =
              _resultSet.getLong("stateContentMasterChangeSeqNum")
          val tmp_stateContentLocalChangeSeqNum =
              _resultSet.getLong("stateContentLocalChangeSeqNum")
          val tmp_stateContentLastChangedBy = _resultSet.getInt("stateContentLastChangedBy")
          val tmp_stateContentLct = _resultSet.getLong("stateContentLct")
          val _entity = StateContentEntity()
          _entity.stateContentUid = tmp_stateContentUid
          _entity.stateContentStateUid = tmp_stateContentStateUid
          _entity.stateContentKey = tmp_stateContentKey
          _entity.stateContentValue = tmp_stateContentValue
          _entity.isIsactive = tmp_isIsactive
          _entity.stateContentMasterChangeSeqNum = tmp_stateContentMasterChangeSeqNum
          _entity.stateContentLocalChangeSeqNum = tmp_stateContentLocalChangeSeqNum
          _entity.stateContentLastChangedBy = tmp_stateContentLastChangedBy
          _entity.stateContentLct = tmp_stateContentLct
          _result.add(_entity)
        }
      }
    }
    return _result
  }

  public override fun findStateContentByKeyAndStateUid(key: String, stateUid: Long):
      StateContentEntity? {
    var _result = null as com.ustadmobile.lib.db.entities.StateContentEntity??
    val _stmtConfig =
        PreparedStatementConfig("SELECT * FROM StateContentEntity WHERE stateContentStateUid = ? AND stateContentKey = ? AND isIsactive"
        , postgreSql = """
    |SELECT * FROM StateContentEntity WHERE stateContentStateUid = ? AND stateContentKey = ? AND isIsactive
    |""".trimMargin())
    _db.prepareAndUseStatement(_stmtConfig) {
      _stmt ->
      _stmt.setLong(1, stateUid)
      _stmt.setString(2, key)
      _stmt.executeQuery().useResults {
         _resultSet ->
        if(_resultSet.next()) {
          val tmp_stateContentUid = _resultSet.getLong("stateContentUid")
          val tmp_stateContentStateUid = _resultSet.getLong("stateContentStateUid")
          val tmp_stateContentKey = _resultSet.getString("stateContentKey")
          val tmp_stateContentValue = _resultSet.getString("stateContentValue")
          val tmp_isIsactive = _resultSet.getBoolean("isIsactive")
          val tmp_stateContentMasterChangeSeqNum =
              _resultSet.getLong("stateContentMasterChangeSeqNum")
          val tmp_stateContentLocalChangeSeqNum =
              _resultSet.getLong("stateContentLocalChangeSeqNum")
          val tmp_stateContentLastChangedBy = _resultSet.getInt("stateContentLastChangedBy")
          val tmp_stateContentLct = _resultSet.getLong("stateContentLct")
          val _entity = StateContentEntity()
          _entity.stateContentUid = tmp_stateContentUid
          _entity.stateContentStateUid = tmp_stateContentStateUid
          _entity.stateContentKey = tmp_stateContentKey
          _entity.stateContentValue = tmp_stateContentValue
          _entity.isIsactive = tmp_isIsactive
          _entity.stateContentMasterChangeSeqNum = tmp_stateContentMasterChangeSeqNum
          _entity.stateContentLocalChangeSeqNum = tmp_stateContentLocalChangeSeqNum
          _entity.stateContentLastChangedBy = tmp_stateContentLastChangedBy
          _entity.stateContentLct = tmp_stateContentLct
          _result = _entity
        }
      }
    }
    return _result
  }

  public override fun setInActiveStateContentByKeyAndUid(
    isActive: Boolean,
    stateUid: Long,
    updateTime: Long
  ): Unit {
    val _stmtConfig = PreparedStatementConfig("""
    |
    |        UPDATE StateContentEntity 
    |           SET isIsactive = ?,  
    |               stateContentLct = ?
    |         WHERE stateContentUid = ?
    |    
    """.trimMargin() , postgreSql = """
    |
    |        UPDATE StateContentEntity 
    |           SET isIsactive = ?,  
    |               stateContentLct = ?
    |         WHERE stateContentUid = ?
    |    
    |""".trimMargin())
    _db.prepareAndUseStatement(_stmtConfig) {
      _stmt ->
      _stmt.setBoolean(1, isActive)
      _stmt.setLong(2, updateTime)
      _stmt.setLong(3, stateUid)
      val _numUpdates = _stmt.executeUpdate()
    }
  }

  public override fun insert(entity: StateContentEntity): Long {
    val _retVal = _insertAdapterStateContentEntity_.insertAndReturnId(entity)
    return _retVal
  }

  public override suspend fun insertAsync(entity: StateContentEntity): Long {
    val _retVal = _insertAdapterStateContentEntity_.insertAndReturnIdAsync(entity)
    return _retVal
  }

  public override fun insertList(entityList: List<out StateContentEntity>): Unit {
    _insertAdapterStateContentEntity_.insertList(entityList)
  }

  public override fun updateList(entityList: List<out StateContentEntity>): Unit {
    val _sql =
        "UPDATE StateContentEntity SET stateContentStateUid = ?, stateContentKey = ?, stateContentValue = ?, isIsactive = ?, stateContentMasterChangeSeqNum = ?, stateContentLocalChangeSeqNum = ?, stateContentLastChangedBy = ?, stateContentLct = ? WHERE stateContentUid = ?"
    _db.prepareAndUseStatement(_sql) {
       _stmt ->
      _stmt.getConnection().setAutoCommit(false)
      for(_entity in entityList) {
        _stmt.setLong(1, _entity.stateContentStateUid)
        _stmt.setString(2, _entity.stateContentKey)
        _stmt.setString(3, _entity.stateContentValue)
        _stmt.setBoolean(4, _entity.isIsactive)
        _stmt.setLong(5, _entity.stateContentMasterChangeSeqNum)
        _stmt.setLong(6, _entity.stateContentLocalChangeSeqNum)
        _stmt.setInt(7, _entity.stateContentLastChangedBy)
        _stmt.setLong(8, _entity.stateContentLct)
        _stmt.setLong(9, _entity.stateContentUid)
        _stmt.executeUpdate()
      }
      _stmt.getConnection().commit()
    }
  }

  public override fun update(entity: StateContentEntity): Unit {
    val _sql =
        "UPDATE StateContentEntity SET stateContentStateUid = ?, stateContentKey = ?, stateContentValue = ?, isIsactive = ?, stateContentMasterChangeSeqNum = ?, stateContentLocalChangeSeqNum = ?, stateContentLastChangedBy = ?, stateContentLct = ? WHERE stateContentUid = ?"
    _db.prepareAndUseStatement(_sql) {
       _stmt ->
      _stmt.setLong(1, entity.stateContentStateUid)
      _stmt.setString(2, entity.stateContentKey)
      _stmt.setString(3, entity.stateContentValue)
      _stmt.setBoolean(4, entity.isIsactive)
      _stmt.setLong(5, entity.stateContentMasterChangeSeqNum)
      _stmt.setLong(6, entity.stateContentLocalChangeSeqNum)
      _stmt.setInt(7, entity.stateContentLastChangedBy)
      _stmt.setLong(8, entity.stateContentLct)
      _stmt.setLong(9, entity.stateContentUid)
      _stmt.executeUpdate()
    }
  }
}
